package com.huimv.center.listener;

import com.alibaba.fastjson.JSONObject;
import com.huimv.center.service.*;
import com.huimv.center.service.impl.BizRawDataServiceImpl;
import com.huimv.center.utils.Const;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class DeviceListener {
    @Autowired
    private BizDeviceRegisteredFlowService deviceRegisteredFlowService;
    @Autowired
    private BizDeviceRegisterService deviceRegisterService;
    @Autowired
    private BizDeviceAnswerFlowService deviceAnswerFlowService;
    @Autowired
    private BizDeviceUnregisteredService deviceUnregisteredService;
    @Autowired
    private BizDeviceUnregisteredFlowService deviceUnregisteredFlowService;
    @Autowired
    private BizRawDataServiceImpl bizRawDataServiceImpl;

    /**
     * @Method      : processDeviceRegisteredFlow
     * @Description : 监听已注册设备
     * @Params      : [askMap]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/6/22
     * @Time        : 20:11
     */
    @RabbitListener(queues = Const.QUEUE_DEVICE_REGISTERED)
    @RabbitHandler
    @Transactional
    public void processDeviceRegisteredFlow(Map askMap) throws ParseException {
        String chipId = askMap.get("chipId").toString();
        String deviceCode = askMap.get("deviceCode").toString();
        String clientIp = askMap.get("clientIp").toString();
        String farmCode = askMap.get("farmCode").toString();
        String farmName = askMap.get("farmName").toString();
        // 添加已注册流水
        deviceRegisteredFlowService.addRegisteredFlow(chipId,deviceCode,clientIp,farmCode,farmName);
        // 更新已注册设备信息
        deviceRegisterService.updateDeviceRegisterTimes(chipId);
    }

    /**
     * @Method      : processDeviceUnregisteredFlow
     * @Description : 监听未注册设备
     * @Params      : [askMap]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/22       
     * @Time        : 20:13
     */
    @RabbitListener(queues = Const.QUEUE_DEVICE_UNREGISTERED)
    @RabbitHandler
    @Transactional
    public void processDeviceUnregisteredFlow(Map askMap) throws ParseException {
        String askText = askMap.get("deviceUnregistered").toString();
        String chipId = askMap.get("deviceUnregistered").toString();
        // 添加非注册流水（统计次数）
        deviceUnregisteredFlowService.addDeviceUnregisteredFlow(chipId);
        // 添加非注册信息
        deviceUnregisteredService.updateDeviceRegisterTimes(chipId);
    }

    /**
     * @Method      : processDeviceAnswer
     * @Description : 监听设备应答
     * @Params      : [askMap]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/6/22
     * @Time        : 20:15
     */
    @RabbitListener(queues = Const.QUEUE_DEVICE_ANSWER)
    @RabbitHandler
    @Transactional
    public void processDeviceAnswer(Map askMap) throws ParseException {
        String askText = askMap.get("deviceAnswer").toString();
        String chipId = askMap.get("chipId").toString();
        String farmCode = askMap.get("farmCode").toString();
        String farmName = askMap.get("farmName").toString();
        String deviceCode = askMap.get("deviceCode").toString();
        // 添加设备应答信息流水信息
        deviceAnswerFlowService.addDeviceAnswer(chipId,deviceCode,askText,farmCode);
        // 计算应答次数
        deviceRegisterService.updateDeviceAnswerTimes(chipId);
    }

    /**
     * @Method      : processAskCmd
     * @Description : 处理原始数据
     * @Params      : [askMap]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/30       
     * @Time        : 18:11
     */
    @RabbitListener(queues = Const.QUEUE_ASK_CMD)
    @RabbitHandler
    public void processAskCmd(Map askMap) throws ParseException {
        String askCmd = askMap.get("askCmd").toString();
        // 添加原始数据信息
        bizRawDataServiceImpl.addRawData(askCmd);
    }

    @RabbitListener(queues = Const.QUEUE_ASK_CMD)
    @RabbitHandler
    public void processSyncFarmDeviceRegister(Map askMap) throws ParseException {
        // 添加原始数据信息
        bizRawDataServiceImpl.syncFarmDeviceRegister(askMap);
    }
}
