package com.huimv.center.producer;

import com.alibaba.fastjson.JSONObject;
import com.huimv.center.utils.Const;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class Producer {
    @Autowired
    RabbitTemplate rabbitTemplate;  //使用RabbitTemplate,这提供了接收/发送等等方法

    //发送已注册消息
    public String sendDeviceRegistered(String idCode, Map resultMap,String registerText, String clientIp) {
        Map map = new HashMap();
        map.put("chipId",idCode);
        map.put("deviceCode",resultMap.get("deviceCode").toString());
        map.put("farmCode",resultMap.get("farmCode").toString());
        map.put("farmName",resultMap.get("farmName").toString());
        map.put("deviceRegistered",registerText);
        map.put("clientIp",clientIp);
        rabbitTemplate.convertAndSend(Const.EXCHANGE_DEVICE_REGISTERED, Const.ROUTING_KEY_DEVICE_REGISTERED, map);
        System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 发送设备已注册消息 >>"+registerText);
        return "ok";
    }

    //发送未注册消息
    public String sendDeviceUnregistered(String idCode,String unregisterText) {
        Map map = new HashMap();
        map.put("deviceUnregistered",unregisterText);
        map.put("chipId",idCode);
        rabbitTemplate.convertAndSend(Const.EXCHANGE_DEVICE_UNREGISTERED, Const.ROUTING_KEY_DEVICE_UNREGISTERED, map);
        System.out.println("--------------------------------------- 发送设备未注册消息 >>"+unregisterText);
        return "ok";
    }

    // 发送应答消息
    public String sendDeviceAnswer(String idCode, Map resultMap,  String answerText) {
        Map map = new HashMap();
        map.put("chipId",idCode);
        map.put("deviceCode",resultMap.get("deviceCode").toString());
        map.put("farmCode",resultMap.get("farmCode").toString());
        map.put("farmName",resultMap.get("farmName").toString());
        map.put("deviceAnswer",answerText);
        rabbitTemplate.convertAndSend(Const.EXCHANGE_DEVICE_ANSWER, Const.ROUTING_KEY_DEVICE_ANSWER, map);
        System.out.println("--------------------------------------- 发送设备应答消息 >>"+answerText);
        return "ok";
    }

    // 保存原始数据
    public String sendAskCmdRawData(String askCmd) {
        Map map = new HashMap();
        map.put("askCmd",askCmd);
        rabbitTemplate.convertAndSend(Const.EXCHANGE_ASK_CMD, Const.ROUTING_KEY_ASK_CMD, map);
        System.out.println("--------------------------------------- 发送设备请求原始数据 >>"+askCmd);
        return "ok";
    }

    // 同步注册信息

    /**
     * 广播同步注册信息
     */
    public String fanoutSyncDevice(Map<String,String> resultMap){
        rabbitTemplate.convertAndSend(Const.QUEUE_SYNC_DEVICE_FANOUT,"",resultMap);
        System.out.println("---------------------------------------广播发送同步设备信息数据"+resultMap);
        return "ok";
    }
}
