package com.huimv.center.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.center.pojo.BizDeviceRegisteredFlow;
import com.huimv.center.mapper.BizDeviceRegisteredFlowMapper;
import com.huimv.center.service.BizDeviceRegisteredFlowService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.center.utils.IDUtil;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-06-22
 */
@Service
public class BizDeviceRegisteredFlowServiceImpl extends ServiceImpl<BizDeviceRegisteredFlowMapper, BizDeviceRegisteredFlow> implements BizDeviceRegisteredFlowService {
    @Autowired
    private BizDeviceRegisteredFlowMapper deviceRegisteredFlowMapper;
    @Autowired
    private com.huimv.center.utils.ResultUtil ResultUtil;
    @Autowired
    private IDUtil idUtil;
    @Autowired
    private com.huimv.center.utils.DateUtil dateUtil;

    /**
     * @Method      : addRegisteredFlow
     * @Description : 添加流水
     * @Params      : [chipId, deviceCode, clientIp]
     * @Return      : com.huimv.eartag2.common.utils.Result
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23
     * @Time        : 17:07
     */
    @Override
    public Result addRegisteredFlow(String chipId, String deviceCode, String clientIp, String farmCode, String farmName) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        BizDeviceRegisteredFlow deviceRegisteredFlow = new BizDeviceRegisteredFlow();
        deviceRegisteredFlow.setChipId(chipId);
        deviceRegisteredFlow.setDeviceCode(deviceCode);
        deviceRegisteredFlow.setClientIp(clientIp);
        deviceRegisteredFlow.setAddTime(dateUtil.getTodayDatetime());
        deviceRegisteredFlow.setFarmCode(farmCode);
        int rows = deviceRegisteredFlowMapper.insert(deviceRegisteredFlow);
        return ResultUtil.result(rows);
    }

    /**
     * @Method      : removeRegisteredFlow
     * @Description : 
     * @Params      : [ids]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 17:29
     */
    @Override
    public Result removeDeviceRegisteredFlow(String ids){
        int rows = deviceRegisteredFlowMapper.deleteBatchIds(idUtil.convert(ids));
        return ResultUtil.result(rows);
    }

    /**
     * @Method      : listDeviceRegisteredFlow
     * @Description : 
     * @Params      : [pageNo, pageSize, chipId]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 18:01
     */
    @Override
    public Result listDeviceRegisteredFlow(Integer pageNo, Integer pageSize, String chipId, String deviceCode){
        if(pageNo == 0){
            pageNo = 1;
        }
        Page<BizDeviceRegisteredFlow> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceRegisteredFlow> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.like(StringUtils.isNotBlank(deviceCode),"device_code", deviceCode);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceRegisteredFlow> pageBizDeviceRegisteredFlow = deviceRegisteredFlowMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceRegisteredFlow);
    }

    /**
     * @Method      : listDeviceRegisteredFlow
     * @Description : 
     * @Params      : [pageNo, pageSize, farmCode, chipId, deviceCode, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/26       
     * @Time        : 20:41
     */
    @Override
    public Result listDeviceRegisteredFlow(Integer pageNo, Integer pageSize, String farmCode, String chipId, String deviceCode, String startDate, String endDate) {
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        Page<BizDeviceRegisteredFlow> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceRegisteredFlow> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(farmCode),"farm_code", farmCode);
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.like(StringUtils.isNotBlank(deviceCode),"device_code", deviceCode);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceRegisteredFlow> pageBizDeviceRegisteredFlow = deviceRegisteredFlowMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceRegisteredFlow);
    }

    /**
     * @Method      : removeDeviceRegisteredFlow
     * @Description : 
     * @Params      : [farmCode, chipId, deviceCode, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/26       
     * @Time        : 21:01
     */
    @Override
    public Result removeDeviceRegisteredFlow(String farmCode, String chipId, String deviceCode, String startDate, String endDate) {
        QueryWrapper<BizDeviceRegisteredFlow> queryWrapper = new QueryWrapper<>();
//        queryWrapper.like(StringUtils.isNotBlank(farmCode),"farm_code", farmCode);
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.like(StringUtils.isNotBlank(deviceCode),"device_code", deviceCode);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);
        return new Result(ResultCode.SUCCESS, deviceRegisteredFlowMapper.delete(queryWrapper));
    }

}
