package com.huimv.center.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.center.pojo.BizDeviceUnregisteredFlow;
import com.huimv.center.mapper.BizDeviceUnregisteredFlowMapper;
import com.huimv.center.service.BizDeviceUnregisteredFlowService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.center.utils.IDUtil;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-06-22
 */
@Service
public class BizDeviceUnregisteredFlowServiceImpl extends ServiceImpl<BizDeviceUnregisteredFlowMapper, BizDeviceUnregisteredFlow> implements BizDeviceUnregisteredFlowService {
    @Autowired
    private BizDeviceUnregisteredFlowMapper deviceUnregisteredFlowMapper;
    @Autowired
    private com.huimv.center.utils.ResultUtil ResultUtil;
    @Autowired
    private IDUtil idUtil;
    @Autowired
    private com.huimv.center.utils.DateUtil dateUtil;

    /**
     * @Method      : addDeviceUnregisteredFlow
     * @Description : 
     * @Params      : [chipId]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 20:22
     */
    @Override
    public Result addDeviceUnregisteredFlow(String chipId) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        BizDeviceUnregisteredFlow deviceUnregisteredFlow = new BizDeviceUnregisteredFlow();
        deviceUnregisteredFlow.setChipId(chipId);
        deviceUnregisteredFlow.setAddTime(dateUtil.getTodayDatetime());
        int rows = deviceUnregisteredFlowMapper.insert(deviceUnregisteredFlow);
        return ResultUtil.result(rows);
    }

    /**
     * @Method      : listDeviceUnregisteredFlow
     * @Description : 
     * @Params      : [pageNo, pageSize, chipId]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 20:22
     */
    @Override
    public Result listDeviceUnregisteredFlow(Integer pageNo, Integer pageSize, String chipId){
        if(pageNo == 0){
            pageNo = 1;
        }
        Page<BizDeviceUnregisteredFlow> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceUnregisteredFlow> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceUnregisteredFlow> pageBizDeviceUnregisteredFlow = deviceUnregisteredFlowMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceUnregisteredFlow);
    }

    /**
     * @Method      : removeByChipIdAndLastTime
     * @Description : 
     * @Params      : [chipId, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/26       
     * @Time        : 16:52
     */
    @Override
    public Result removeByChipIdAndLastTime(String farmCode,String chipId, String startDate, String endDate) {
        QueryWrapper<BizDeviceUnregisteredFlow> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);
        return new Result(ResultCode.SUCCESS, deviceUnregisteredFlowMapper.delete(queryWrapper));
    }

    /**
     * @Method      : listByFarmCode
     * @Description : 
     * @Params      : [pageNo, pageSize, farmCode, chipId, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/26       
     * @Time        : 16:52
     */
    @Override
    public Result listByFarmCode(Integer pageNo, Integer pageSize, String chipId, String startDate, String endDate) {
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        Page<BizDeviceUnregisteredFlow> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceUnregisteredFlow> queryWrapper = new QueryWrapper<>();
//        queryWrapper.like(StringUtils.isNotBlank(farmCode),"farm_code", farmCode);
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);

        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceUnregisteredFlow> pageBizDeviceUnregisteredFlow = deviceUnregisteredFlowMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceUnregisteredFlow);
    }

    /**
     * @Method      : listChipId
     * @Description : 
     * @Params      : [pageNo, pageSize, chipId, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/26       
     * @Time        : 16:52
     */
    @Override
    public Result listChipId(Integer pageNo, Integer pageSize, String chipId, String startDate, String endDate) {
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        Page<BizDeviceUnregisteredFlow> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceUnregisteredFlow> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceUnregisteredFlow> pageBizDeviceUnregisteredFlow = deviceUnregisteredFlowMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceUnregisteredFlow);
    }

    @Override
    public Result removeById(String ids) {
        return ResultUtil.result(deviceUnregisteredFlowMapper.deleteBatchIds(idUtil.convert(ids)));
    }
}
