package com.huimv.center.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.center.pojo.BizDeviceUnregistered;
import com.huimv.center.mapper.BizDeviceUnregisteredMapper;
import com.huimv.center.service.BizDeviceUnregisteredService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.center.utils.IDUtil;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-06-22
 */
@Service
public class BizDeviceUnregisteredServiceImpl extends ServiceImpl<BizDeviceUnregisteredMapper, BizDeviceUnregistered> implements BizDeviceUnregisteredService {
    @Autowired
    private BizDeviceUnregisteredMapper deviceUnregisteredMapper;
    @Autowired
    private com.huimv.center.utils.ResultUtil ResultUtil;
    @Autowired
    private IDUtil idUtil;
    @Autowired
    private com.huimv.center.utils.DateUtil dateUtil;

    /**
     * @Method      : updateDeviceRegisterTimes
     * @Description : 
     * @Params      : [chipId]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 20:16
     */
    @Override
    public Result updateDeviceRegisterTimes(String chipId) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        QueryWrapper<BizDeviceUnregistered> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("chip_id", chipId);
        // 查询注册设备
        BizDeviceUnregistered deviceRegister = deviceUnregisteredMapper.selectOne(queryWrapper);
        int rows = 0;
        if(deviceRegister == null){
            BizDeviceUnregistered newDeviceUnregistered = new BizDeviceUnregistered();
            newDeviceUnregistered.setChipId(chipId);
            newDeviceUnregistered.setAskTimes(1);
            newDeviceUnregistered.setFirstTime(dateUtil.getTodayDatetime());
            newDeviceUnregistered.setLastTime(dateUtil.getTodayDatetime());
            rows = deviceUnregisteredMapper.insert(newDeviceUnregistered);
        }else{
            Integer times = deviceRegister.getAskTimes();
            if(times == null){
                times = 0;
            }
            deviceRegister.setAskTimes(++times);
            deviceRegister.setLastTime(dateUtil.getTodayDatetime());
            rows = deviceUnregisteredMapper.updateById(deviceRegister);
        }
        return ResultUtil.result(rows);
    }

    /**
     * @Method      : listDeviceUnregistered
     * @Description : 
     * @Params      : [pageNo, pageSize, chipId]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 20:22
     */
    @Override
    public Result listDeviceUnregistered(Integer pageNo,Integer pageSize,String chipId,String startDate,String endDate){
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        Page<BizDeviceUnregistered> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizDeviceUnregistered> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(chipId),"chip_id", chipId);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"last_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"last_time", endDate);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizDeviceUnregistered> pageBizDeviceUnregistered = deviceUnregisteredMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizDeviceUnregistered);
    }

    @Override
    public Result remove(String ids) {
        return new Result(ResultCode.SUCCESS, deviceUnregisteredMapper.deleteBatchIds(idUtil.convert(ids)));
    }

}
