package com.huimv.center.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.center.pojo.BizBaseArea;
import com.huimv.center.pojo.BizDeviceRegister;
import com.huimv.center.pojo.BizFarm;
import com.huimv.center.mapper.BizFarmMapper;
import com.huimv.center.pojo.BizRawData;
import com.huimv.center.service.BizBaseAreaService;
import com.huimv.center.service.BizFarmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-06-21
 */
@Service
public class BizFarmServiceImpl extends ServiceImpl<BizFarmMapper, BizFarm> implements BizFarmService {
    @Autowired
    private BizFarmMapper BizFarmMapper;
    private BizFarm farm;
    @Autowired
    private com.huimv.center.utils.IDUtil idUtil;
    @Autowired
    private com.huimv.center.utils.DateUtil dateUtil;
    @Autowired
    private com.huimv.center.utils.ResultUtil ResultUtil;

    @Resource
    private BizBaseAreaService bizBaseAreaService;
    /**
     * @Method      : save
     * @Description : 
     * @Params      : [farmName, farmCode, linkman, phone, remark]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/24       
     * @Time        : 14:46
     */
    @Override
    public Result save(String farmName, String farmCode, String linkman, String phone, String remark,String areaName) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        BizFarm farm = new BizFarm();
        farm.setFarmName(farmName);
        farm.setFarmCode(farmCode);
        farm.setLinkman(linkman);
        farm.setPhone(phone);
        farm.setRemark(remark);
        farm.setAddTime(dateUtil.getTodayDatetime());
        //根据区域名称查询区域编码
        BizBaseArea bizBaseArea = bizBaseAreaService.queryAreaCodeByAreaName(areaName);
        if (Objects.nonNull(bizBaseArea)){
            farm.setAreaCode(bizBaseArea.getAreaCode());
        }
        farm.setAreaName(areaName);
        return ResultUtil.result(BizFarmMapper.insert(farm));
    }

    /**
     * @Method      : edit
     * @Description : 
     * @Params      : [id, farmName, farmCode, linkman, phone, remark]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/24       
     * @Time        : 14:44
     */
    @Override
    public Result edit(Integer id, String farmName, String farmCode, String linkman, String phone, String remark,String areaName) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        BizFarm farm = BizFarmMapper.selectById(id);
        farm.setFarmName(farmName);
        farm.setFarmCode(farmCode);
        farm.setLinkman(linkman);
        farm.setPhone(phone);
        farm.setRemark(remark);
        farm.setAddTime(dateUtil.getTodayDatetime());
        //根据区域名称查询区域编码
        BizBaseArea bizBaseArea = bizBaseAreaService.queryAreaCodeByAreaName(areaName);
        if (Objects.nonNull(bizBaseArea)){
            farm.setAreaCode(bizBaseArea.getAreaCode());
        }
        farm.setAreaName(areaName);
        return ResultUtil.result(BizFarmMapper.updateById(farm));
    }

    /**
     * @Method      : remove
     * @Description : 
     * @Params      : [ids]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/24       
     * @Time        : 14:44
     */
    @Override
    public Result remove(String ids){
        return ResultUtil.result(BizFarmMapper.deleteBatchIds(idUtil.convert(ids)));
    }

    /**
     * @Method      : list
     * @Description : 
     * @Params      : [pageNo, pageSize, farmName, farmCode, startDate, endDate]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/24       
     * @Time        : 14:44
     */
    @Override
    public Result list(Integer pageNo, Integer pageSize, String farmName,String farmCode,String startDate,String endDate){
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        Page<BizFarm> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizFarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(farmName),"farm_name", farmName);
        queryWrapper.like(StringUtils.isNotBlank(farmCode),"farm_code", farmCode);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time", startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time", endDate);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizFarm> pageBizFarm = BizFarmMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizFarm);
    }

    @Override
    public String getBizFarmByFarmCode(String farmCode) {
        QueryWrapper<BizFarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", farmCode);
        List<BizFarm> bizFarms = BizFarmMapper.selectList(queryWrapper);
        if (bizFarms == null || bizFarms.size()==0){
            System.out.println("----------------牧场表里面查不到这条数据");
        }else{
            return bizFarms.get(0).getAreaCode();
        }
        return "";
    }
}
