package com.huimv.center.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.center.pojo.BizDeviceUnregisteredFlow;
import com.huimv.center.pojo.BizRawData;
import com.huimv.center.mapper.BizRawDataMapper;
import com.huimv.center.service.BizRawDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-06-23
 */
@Service
public class BizRawDataServiceImpl extends ServiceImpl<BizRawDataMapper, BizRawData> implements BizRawDataService {
    @Autowired
    private BizRawDataMapper BizRawDataMapper;
    @Autowired
    private com.huimv.center.utils.ResultUtil ResultUtil;
    @Autowired
    private com.huimv.center.utils.DateUtil dateUtil;
    @Autowired
    private com.huimv.center.utils.IDUtil idUtil;

    /**
     * @Method      : addRawData
     * @Description : 
     * @Params      : [askCmd]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 21:04
     */
    @Override
    public Result addRawData(String askCmd) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        BizRawData bizRawData = new BizRawData();
        bizRawData.setAskCmd(askCmd);
        bizRawData.setAddTime(dateUtil.getTodayDatetime());
        int rows = BizRawDataMapper.insert(bizRawData);
        return ResultUtil.result(rows);
    }

    /**
     * @Method      : listRawData
     * @Description : 
     * @Params      : [pageNo, pageSize, searchText]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/23       
     * @Time        : 21:04
     */
    @Override
    public Result listRawData(Integer pageNo, Integer pageSize, String searchText,String startDate,String endDate){
        if(pageNo == 0){
            pageNo = 1;
        }
        startDate = dateUtil.buildStartDate(startDate);
        endDate = dateUtil.buildEndDate(endDate);
        System.out.println("startDate>>"+startDate);
        System.out.println("endDate>>"+endDate);
        Page<BizRawData> page = new Page<>(pageNo, pageSize);
        QueryWrapper<BizRawData> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(searchText),"ask_cmd", searchText);
        queryWrapper.ge(StringUtils.isNotBlank(startDate),"add_time",startDate);
        queryWrapper.le(StringUtils.isNotBlank(endDate),"add_time",endDate);
        //创建时间降序
        queryWrapper.orderByDesc("id");
        Page<BizRawData> pageBizRawData = BizRawDataMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,pageBizRawData);
    }

    @Override
    public Result remove(String ids){
        return ResultUtil.result(BizRawDataMapper.deleteBatchIds(idUtil.convert(ids)));
    }

    /**
     * @Method      : syncFarmDeviceRegister
     * @Description : 同步牧场基站设备注册
     * @Params      : [askMap]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/6/30       
     * @Time        : 18:15
     */
    @Override
    public void syncFarmDeviceRegister(Map askMap) {
        String askCmd = askMap.get("askCmd").toString();

        //注册牧场信息

        //注册基站设备信息

    }
}
